import peasy.*;
import processing.opengl.*;
import javax.media.opengl.*;
import controlP5.*;

PShader texture_shader;
PShape slices, slices_r;
PShape frame;
PFont font;

PeasyCam cam;
ControlP5 controlP5;

int swipe_mode = 0;
int text_mode = 1;

void setup() {
  size(1200, 800, P3D);
  smooth(8);
  
  controlP5 = new ControlP5(this);
  
  controlP5.addToggle("Swipe", false, 70, 10, 60, 20);
  controlP5.setAutoDraw(false);
  
  font = createFont("Helvetica", 72, false);
  
  cam = new PeasyCam(this, 200);
  cam.setMinimumDistance(100);
  cam.setMaximumDistance(500);
  
  texture_shader = loadShader("shader.glsl", "vertex.glsl");
  slices = createSlices(false);
  slices_r = createSlices(true);
  //frame = createFrame();
}

PShape createSlices(boolean reversed) {
  textureMode(IMAGE);
  PShape sh = createShape();
  sh.beginShape(QUADS);
  sh.noStroke();
  sh.texture(loadImage("flatten.png"));
  if(!reversed) {
    for(int i = 0; i < 12 * 9; i++) {
      int x = i % 11;
      int y = i / 11;
      float u = x * 256;
      float v = y * 256;
      float z = 120 * ((float)i / 108 - 0.5);
      sh.normal(0, 0, 1);
      sh.vertex(-100, -100, z, u, v);
      sh.vertex(-100, +100, z, u, v + 256);
      sh.vertex(+100, +100, z, u + 256, v + 256);
      sh.vertex(+100, -100, z, u + 256, v);
    }
  } else {
    for(int i = 12 * 9 - 1; i >= 0; i--) {
      int x = i % 11;
      int y = i / 11;
      float u = x * 256;
      float v = y * 256;
      float z = 120 * ((float)i / 108 - 0.5);
      sh.normal(0, 0, 1);
      sh.vertex(-100, -100, z, u, v);
      sh.vertex(-100, +100, z, u, v + 256);
      sh.vertex(+100, +100, z, u + 256, v + 256);
      sh.vertex(+100, -100, z, u + 256, v);
    }
  }
  sh.endShape();
  return sh;
}

void draw() {
  float time = millis();
  background(0);
  textAlign(LEFT, BOTTOM);
  hint(DISABLE_DEPTH_TEST);
  shader(texture_shader);
  float z_center = 120.0 * sin(time / 1000.0) / 2.0;
  texture_shader.set("swipe_mode", swipe_mode);
  texture_shader.set("z_center", z_center);
  float direction_x = cam.getLookAt()[0] - cam.getPosition()[0];
  float direction_y = cam.getLookAt()[1] - cam.getPosition()[1];
  float direction_z = cam.getLookAt()[2] - cam.getPosition()[2];
  if(direction_z < 0) {
    shape(slices);
  } else {
    shape(slices_r);
  }
  resetShader();
  
  stroke(255, 255, 255, 64);
  noFill();
  box(150, 150, 120);
  for(int year = 0; year < 9; year += 1) {
    for(int month = 0; month < 12; month += 1) {
      float i = year * 12 + month;
      float z = 120 * ((float)i / 108 - 0.5);
      if(month == 0) {
        line(-75, -75, z, 75, -75, z);
      } else {
        line(-75, -75, z, -70, -75, z);
      }
    }
  }
  fill(255, 255, 255, 192);
  noStroke();
  textFont(font, 12);
  for(int year = 0; year < 9; year += 1) {
    for(int month = 0; month < 12; month += 1) {
      if(month != 0) continue;
      float i = year * 12 + month;
      float z = 120 * ((float)i / 108 - 0.5);
      pushMatrix();
      translate(-75, -75, z);
      rotateX(-PI / 2);
      text(year + 2006, 10, 0, 0);
      popMatrix();
    }
  }
  
  textFont(font, 24);
  for(int i = 0; i < labels.length; i++) {
    float x = Float.parseFloat(labels[i][0]);
    float y = Float.parseFloat(labels[i][1]);
    x = (x - (-15)) / (20 - (-15)) * 200 - 100;
    y = (y - (-15)) / (15 - (-15)) * 200 - 100;
    String s = labels[i][2].toUpperCase();
    pushMatrix();
    if(text_mode == 0) {
      translate(x, y, 60);
      rotateY(PI / 2);
      textAlign(RIGHT, CENTER);
      scale(0.1);
      text(s, 0, 0, 0);
    } else {
      translate(x, y, 0);
      rotateY(PI / 2);
      rotateX(PI);
      if(cam.getPosition()[0] > x) {
        rotateY(PI);
      }
      rotateZ(PI);
      textAlign(CENTER, CENTER);
      scale(0.15);
      text(s, 0, 0, 0);
    }
    popMatrix();
  }
  
  noStroke();
  noFill();
  
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  controlP5.draw();
  cam.endHUD();
}

void controlEvent(ControlEvent theEvent) {
  if(theEvent.isController()) { 
    if(theEvent.controller().name() == "Swipe") {
      if(theEvent.controller().value() == 1) {
        swipe_mode = 1;
      } else {
        swipe_mode = 0;
      }
    }
  }  
}
